<?php
/**
 * Import customize style
 *
 * @return Css inline at header.
 *
 * @package     Zoo Theme
 * @version     1.0.0
 * @author      Zootemplate
 * @link        https://www.zootemplate.com/
 * @copyright   Copyright (c) 2018 Zootemplate
 */

// Render css customize
add_action('wp_enqueue_scripts', 'zoo_enqueue_render', 1000);
// Enqueue scripts for theme.
function zoo_enqueue_render()
{
    // Load custom style
    wp_add_inline_style('zoo-styles', zoo_customize_style());
}
if (!function_exists('zoo_customize_style')) {
    function zoo_customize_style()
    {
        /* ----------------------------------------------------------
                                    Responsive control
                            Control Breakpoint of header Layout
                            Don't remove this section
        ---------------------------------------------------------- */
        $css = '';
        $theme_settings = get_option(ZOO_SETTINGS_KEY, []);
        $mobile_breakpoint = !empty($theme_settings['mobile_breakpoint_width']) ? strval(intval($theme_settings['mobile_breakpoint_width'])) : '992';
        $css .= '@media(min-width: ' . $mobile_breakpoint . 'px) {
                    .wrap-site-header-mobile {
                        display: none;
                    }
                    .show-on-mobile {
                        display: none;
                    }
                }
        
                @media(max-width: ' . $mobile_breakpoint . 'px) {
                    .wrap-site-header-desktop {
                        display: none;
                    }
                    .show-on-desktop {
                        display: none;
                    }
                }
        ';
        $css .= '@media(min-width:1500px){.elementor-section.elementor-section-boxed>.elementor-container,.container{max-width:' . zoo_site_width() . ';width:100%}}';
        /* ----------------------------------------------------------
                            End Responsive control
                    Control Breakpoint of header Layout
                    Don't remove this section
        ---------------------------------------------------------- */
        /* ----------------------------------------------------------
                                    Typography
                            All typography must add here
        ---------------------------------------------------------- */
        $font_name = get_theme_mod('zoo_typo_new_font_family');
        $font_arr = get_theme_mod('zoo_font_items');
        if ($font_name && get_theme_mod('zoo_use_font','default') == 'custom' ) {   
            foreach ($font_arr as $key => $font) {
                $css .= '@font-face {';
                $css .= 'font-family: '.$font_name.';';
                $css .= 'font-weight: '.$font['weight'].';';
                $css .= 'font-style:  '.$font['style'].';';

                $css .= 'src:';
                $arr  = array();
                if ( $font['woff2'] ) {
                    $arr[] = 'url(' . esc_url( $font['woff2'] ) . ") format('woff2')";
                }
                if ( $font['woff'] ) {
                    $arr[] = 'url(' . esc_url( $font['woff'] ) . ") format('woff')";
                }
                if ( $font['ttf'] ) {
                    $arr[] = 'url(' . esc_url( $font['ttf'] ) . ") format('truetype')";
                }
                if ( $font['svg'] ) {
                    $arr[] = 'url(' . esc_url( $font['svg'] ) . '#' . esc_attr( strtolower( str_replace( ' ', '_', $font_name ) ) ) . ") format('svg')";
                }
                $css .= join( ', ', $arr );
                $css .= ';';
                $css .= '}';
            } 
            //display font
            $css .= 'body{';
            $css .= 'font-family: "'.$font_name.'", sans-serif;;';
            $css .= 'font-weight: normal;';
            $css .= 'font-style:  normal;';
            $css .= 'font-size:  16px;';
            $css .= '}';
            //font-size
            $css .= "html{";
            $css .= "font-size: 16px;";
            $css .= "}";
        }
        if( get_theme_mod('zoo_use_font','default') == 'default'){
            $css .= '@font-face {
                        font-family: \'Larsseit\';
                        src: url(\'' . ZOO_THEME_URI.'assets/fonts/larsseit/larsseit-light.woff\') format(\'woff\');
                        font-weight: 300;
                        font-style: normal;
                        font-display: auto;
                    }

                    @font-face {
                        font-family: \'Larsseit\';
                        src: url(\'' . ZOO_THEME_URI.'assets/fonts/larsseit/larsseit.woff\') format(\'woff\');
                        font-weight: 400;
                        font-style: normal;
                        font-display: auto;
                    }

                    @font-face {
                        font-family: \'Larsseit\';
                        src: url(\'' . ZOO_THEME_URI.'assets/fonts/larsseit/larsseit-medium.woff\') format(\'woff\');
                        font-weight: 500;
                        font-style: normal;
                        font-display: auto;
                    }

                    @font-face {
                        font-family: \'Larsseit\';
                        src: url(\'' . ZOO_THEME_URI.'assets/fonts/larsseit/larsseit-bold.woff\') format(\'woff\');
                        font-weight: 600;
                        font-style: normal;
                        font-display: auto;
                    }
                ';
                //display font
                $css .= 'body{';
                $css .= 'font-family: \'Larsseit\', sans-serif;;';
                $css .= 'font-weight: normal;';
                $css .= 'font-style:  normal;';
                $css .= 'font-size:  16px;';
                $css .= "}";
                //font-size
                $css .= "html{";
                $css .= "font-size: 16px;";
                $css .= "}";
        }

        if (class_exists('WooCommerce')) {
            $gutter = zoo_product_gutter();
            $css .= '.products .product{padding-left:' . $gutter . 'px;padding-right:' . $gutter . 'px}';
            $css .= 'ul.products, .woocommerce ul.products{margin-left:-' . $gutter . 'px !important;margin-right:-' . $gutter . 'px !important;width:calc(100% + ' . ($gutter * 2) . 'px)}';
            
            if(get_theme_mod('zoo_enable_product_stock','1')== 0 ){
                $css .= '.stock-quantity,.stock {display: none;}';
            }
            if (get_post_meta(get_the_id(), 'zoo_disable_store_notice', true) == 1) {
                $css .= '.woocommerce-store-notice.demo_store {display: none !important;}';
            }
            
        }
        return $css;
    }
}